from cndi.annotations import Component, ConditionalRendering
from cndi.consts import RCN_AUTO_CONFIGURE_SECRETS_ENABLE
from cndi.env import getContextEnvironment


@Component
@ConditionalRendering(callback=lambda x: getContextEnvironment(RCN_AUTO_CONFIGURE_SECRETS_ENABLE, defaultValue=False, castFunc=bool))
class AutoConfigurationProviders:
    _PROVIDERS = dict()
    def __init__(self):
        pass


import os
from pathlib import Path

from cndi import BASE_NAME


class ResourceFinder:
    def __init__(self):
        if f"{BASE_NAME}_HOME" in os.environ:
            self.rcnHome = os.environ[f'{BASE_NAME}_HOME']
        else:
            self.rcnHome = os.path.join(Path.home().absolute().__str__(), ".rcn")

    def findResource(self, resourcePath):
        currentPath = Path(os.path.abspath(os.path.curdir))
        resourceDirPath = os.path.join(currentPath, "resources")
        resourceExist = os.path.exists(resourceDirPath)

        while not resourceExist \
                and len(resourceDirPath) >= len(self.rcnHome):
            currentPath = currentPath.parent
            resourceDirPath = os.path.join(currentPath.absolute(), "resources")
            resourceExist = os.path.exists(resourceDirPath) and os.path.isdir(resourceDirPath)

        if f'{BASE_NAME}_RESOURCES_DIR' in os.environ and resourceExist == False:
            resourceDirPath = os.environ[f'{BASE_NAME}_RESOURCES_DIR']
            resourceExist = os.path.exists(os.path.join(resourceDirPath,
                                                        resourcePath))

        if resourceExist:
            resourcePath = os.path.join(resourceDirPath, resourcePath)
            if os.path.exists(resourcePath):
                return resourcePath
            raise FileNotFoundError(f"Resource not found at resources/{resourcePath}")
        else:
            raise FileNotFoundError(f"Resource Path not found resources/{resourcePath}")


import logging
from threading import Thread
from typing import List

from cndi.annotations import Component, ConditionalRendering
from cndi.consts import RCN_ENABLE_CONTEXT_THREADS
from cndi.env import getContextEnvironment

log = logging.getLogger(__name__)

def isContextThreadEnable(dependent):
    enabled = getContextEnvironment(RCN_ENABLE_CONTEXT_THREADS, defaultValue=False, castFunc=bool)
    if not enabled:
        log.warning(f"{dependent} Component depends on {__name__}.{ContextThreads.__name__}")
        log.warning(f"Context Threads is disable in property please enable it by setting {RCN_ENABLE_CONTEXT_THREADS} to true")

    return enabled

@Component
@ConditionalRendering(callback=lambda x: getContextEnvironment(RCN_ENABLE_CONTEXT_THREADS, defaultValue=False, castFunc=bool))
class ContextThreads:
    def __init__(self):
        self.threads: List[Thread] = list()

    def add_thread(self, thread):
        self.threads.append(thread)

    def clean_up(self):
        exitedThread = []
        for thread in self.threads:
            if not thread.is_alive():
                exitedThread.append(thread)

        for exitThread in exitedThread:
            self.threads.remove(exitThread)


import logging
import threading

from cndi.annotations import Component, ConditionalRendering, getBeanObject
from cndi.annotations.threads import ContextThreads, isContextThreadEnable
from cndi.consts import RCN_FLASK_APP_ENABLED, RCN_FLASK_APP_NAME, RCN_FLASK_APP_PORT, RCN_FLASK_APP_LISTEN, \
    RCN_FLASK_APP_CONTEXT_URL, RCN_FLASK_APP_CONFIGS_LIST, RCN_FLASK_APP_BACKEND_SERVER
from cndi.env import getContextEnvironment, getContextEnvironments

logger = logging.getLogger(__name__)

def __check_flask_enabled__(x):
    """
    Checks whether Flask is enabled.

    This function checks the environment for a variable named 'app.flask.enabled'. If the variable is set to 'True', the function returns True. If the variable is not set or is set to any other value, the function returns False.

    Returns:
        True if Flask is enabled, False otherwise.
    """
    try:
        from flask import Flask
        return getContextEnvironment(RCN_FLASK_APP_ENABLED, castFunc=bool, defaultValue=False) \
            and isContextThreadEnable(f"{FlaskApplication.__module__}.{FlaskApplication.__qualname__}")
    except ImportError:
        return False

@Component
@ConditionalRendering(callback=__check_flask_enabled__)
class FlaskApplication:
    """
    This class represents a Flask application.

    It is marked as a component and is conditionally rendered based on whether Flask is enabled.

    Attributes:
        appName: The name of the Flask application.
        port: The port on which the Flask application runs.
        host: The host on which the Flask application runs.
        configs: A dictionary of configuration settings for the Flask application.
        app: The Flask application instance.

    Methods:
        postConstruct: A method that is called after the Flask application is constructed.
        run: Starts the Flask application.
    """
    def __init__(self):
        """
        Initializes a new instance of the Flask application.

        The application's name, port, host, debug mode, and configuration settings are retrieved from the environment.
        """
        from flask import Flask, Blueprint
        self.appName = getContextEnvironment(RCN_FLASK_APP_NAME, defaultValue="app")
        self.port = getContextEnvironment(RCN_FLASK_APP_PORT, defaultValue=8080, castFunc=int)
        self.host = getContextEnvironment(RCN_FLASK_APP_LISTEN, defaultValue="0.0.0.0")
        self.contextUrl = getContextEnvironment(RCN_FLASK_APP_CONTEXT_URL, defaultValue="/")
        contextEnvs = getContextEnvironments()
        self.configs = dict(map(lambda items: (items[0][f'{RCN_FLASK_APP_CONFIGS_LIST}.'.__len__():], items[1]),
                                filter(lambda items: items[0].startswith(RCN_FLASK_APP_CONFIGS_LIST), contextEnvs.items())))

        self.__app = Flask(self.appName)
        self.app = Blueprint(self.appName, __name__)
        

    def postConstruct(self):
        """
        A method that is called after the Flask application is constructed.

        This method currently does nothing, but can be overridden in subclasses to perform setup tasks.
        """
        pass

    def run(self):
        """
        Starts the Flask application.

        This method starts the Flask application on the specified host and port.
        """

        self.__app.register_blueprint(self.app, url_prefix=self.contextUrl)

        backend = getContextEnvironment(RCN_FLASK_APP_BACKEND_SERVER, defaultValue="werkzeug")
        logger.info(f"Starting Flask Server at {self.host}:{self.port} at Context URL: {self.contextUrl} on Backend {backend}")

        if backend == "werkzeug":
            from werkzeug import run_simple
            serverThread = threading.Thread(name=f"thread-{self.appName}", target=run_simple, kwargs={
                "hostname": self.host,
                "port": self.port,
                "application": self.__app,
                **self.configs
            })
        elif backend == "waitress":
            from waitress import serve
            serverThread = threading.Thread(name=f"thread-{self.appName}", target=serve, kwargs={
                "host": self.host,
                "port": self.port,
                "app": self.__app,
                **self.configs
            })

        contextThread: ContextThreads = getBeanObject('.'.join([ContextThreads.__module__ , ContextThreads.__name__]))
        contextThread.add_thread(serverThread)

        serverThread.start()


import asyncio
import logging
from concurrent.futures.thread import ThreadPoolExecutor
from functools import wraps
from typing import Dict, Callable

from cndi.annotations import Component, constructKeyWordArguments, ConditionalRendering
from cndi.consts import RCN_EVENTS_ENABLE, RCN_EVENTS_NUM_THREADS
from cndi.env import getContextEnvironment

logger = logging.getLogger(__name__)

class BuiltInEventsTypes:
    ON_ENV_LOAD="on_env_load"
    ON_CONTEXT_LOAD="on_context_load"

class Event(object):
    def __init__(self, eventType,
                 eventCallback, kwargs={}):
        self.eventType = eventType
        self.eventCallback: Callable = eventCallback
        self.kwargs = kwargs

REGISTERED_EVENTS: Dict[str, dict[str, Event]] = dict()
def register_event(event: Event, func_name: str):
    if event.eventType not in REGISTERED_EVENTS:
        REGISTERED_EVENTS[event.eventType] = dict()

    REGISTERED_EVENTS[event.eventType][func_name] = event

def OnEvent(event):
    def inner_function(func):
        annotations = func.__annotations__

        func_name = '.'.join([func.__module__, func.__qualname__])
        @wraps(func)
        def wrapper(*args, **kwargs):
            if asyncio.iscoroutinefunction(func):
                return asyncio.run(func(*args, **kwargs))
            return func(*args, **kwargs)

        register_event(Event(eventType=event, eventCallback=wrapper, kwargs=annotations), func_name)

        return wrapper
    return inner_function

class EventNotFound(Exception):
    def __init__(self, *args):
        super().__init__( *args)

@Component
@ConditionalRendering(callback=lambda x: getContextEnvironment(RCN_EVENTS_ENABLE, defaultValue=False, castFunc=bool))
class EventBus:
    def __init__(self):
        self._executor = ThreadPoolExecutor(max_workers=getContextEnvironment(RCN_EVENTS_NUM_THREADS, defaultValue=1, castFunc=int))

    def publish(self, event_name: str, data: dict = {}) -> None:
        if event_name in REGISTERED_EVENTS:
            for func_name, event  in REGISTERED_EVENTS[event_name].items():
                kwargs = {
                    **constructKeyWordArguments(event.kwargs, required=False),
                    **data
                }
                kwargs = dict(map(lambda x: [x, kwargs[x]], set(event.kwargs.keys()).intersection(kwargs.keys())))
                self._executor.submit(event.eventCallback, **kwargs)
        else:
            logger.warning(f"{event_name} event not found, please check the decorators")

    def subscribe(self, event_name: str, event: Event) -> None:
        if event_name not in REGISTERED_EVENTS:
            REGISTERED_EVENTS[event_name] = dict()
        REGISTERED_EVENTS[event_name][event.eventType] = event


schema = {
    "type": "object",
    "properties": {
        "rcn": {
            "type": "object",
            "properties": {
                "profile": {"type": "string" },
                "binders": {
                    "type": "object",
                    "properties": {
                        "message": {
                            "type": "object"
                        }
                    }
                }
            }
        }
    }
}


import os
from pathlib import Path

import yaml
from yaml import SafeLoader


class BaseConfigGenerator:
    def __init__(self, sourceDir, destinationFile):
        self.sourceDir = sourceDir
        self.destinationFile = destinationFile
    def readDocs(self, exts=['.yaml', '.yml']):
        files = filter(lambda x: os.path.isfile(x) and os.path.splitext(x)[1] in exts,
                       map(
                           lambda x: os.path.join(self.sourceDir, x), os.listdir(self.sourceDir)
                           )
                       )

        docs = []
        for file in files:
            with open(file, 'r') as stream:
                docs.append(yaml.load(stream, Loader=SafeLoader))

        return docs

class NLUConfigGenerator(BaseConfigGenerator):
    def __init__(self, sourceDir: str, destinationFile: str, version='3.1'):
        BaseConfigGenerator.__init__(self, sourceDir, destinationFile)
        self.version = version

    def transform(self):
        docs = self.readDocs()
        outputData = {}
        for doc in docs:
            intent = doc['intent']
            examples = list(doc['examples'].split("\n")) if isinstance(doc['examples'], str) else doc['examples']
            if intent in outputData:
                outputData[intent].extend(examples)
            else:
                outputData[intent] = examples
        return outputData

    def writeToFile(self, outputData={}):
        nlu = []
        for intent, examples in outputData.items():
            example = str.join("", map(lambda x: f"- {x} \n", examples))
            nlu.append({
                "intent": intent,
                "examples": example
            })

        nluData = {
            "version": self.version,
            "nlu": nlu
        }

        with open(self.destinationFile, "w") as  stream:
            yaml.dump(nluData, stream)

        return Path(self.destinationFile)


if __name__ == '__main__':
    nlu = NLUConfigGenerator("./", "output.yml")
    output = nlu.transform()
    print(nlu.writeToFile(output))





import logging
import threading
import time
import uuid
from concurrent.futures.thread import ThreadPoolExecutor
from dataclasses import dataclass
from enum import Enum
from queue import Queue

from cndi.annotations import Component, ConditionalRendering
from cndi.annotations.threads import ContextThreads
from cndi.consts import RCN_ENABLE_CONTEXT_THREADS, RCN_ENABLE_TASK_EXECUTOR, RCN_TASK_EXECUTOR_NUM_THREADS
from cndi.env import getContextEnvironment

logger = logging.getLogger(__name__)

@dataclass
class Task:
    def __init__(self, name: str, callback,  completed_callback=None):
        self.name = name
        self.callback = callback
        self.completed_callback = completed_callback
        self.result = None
        self.id = uuid.uuid4().__str__()
        self.status = TaskStatus.CREATED

class TaskStatus(Enum):
    CREATED = "CREATED"
    SCHEDULED = "SCHEDULED"
    RUNNING = "RUNNING"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"

@Component
@ConditionalRendering(callback=lambda x: getContextEnvironment(RCN_ENABLE_TASK_EXECUTOR, defaultValue=False, castFunc=bool)
                      and getContextEnvironment(RCN_TASK_EXECUTOR_NUM_THREADS, defaultValue=1, castFunc=int) > 0
                      and getContextEnvironment(RCN_ENABLE_CONTEXT_THREADS, defaultValue=False, castFunc=bool))
class TaskExecutor(threading.Thread):
    def __init__(self, context_threads: ContextThreads):
        super().__init__()
        self.tasks = {}
        self.num_threads = getContextEnvironment(RCN_TASK_EXECUTOR_NUM_THREADS, defaultValue=1, castFunc=int)
        self.queue = Queue()
        self.running = True
        context_threads.add_thread(self)
        self.start()

    def add_task(self, task: Task):
        self.tasks[task.id] = task
        self.queue.put(task)
        task.status = TaskStatus.SCHEDULED
        logger.debug(f"Added task {task.name} with id {task.id} to the queue")

    def remove_task(self, name: str):
        if name in self.tasks:
            del self.tasks[name]

    def execute(self, id: str, *args, **kwargs):
        if id in self.tasks:
            return self.tasks[id].callback(*args, **kwargs)
        else:
            raise Exception(f"Task with name {id} not found")

    def get_task(self,id):
        if id in self.tasks:
            return self.tasks[id]
        else:
            raise Exception(f"Task with name {id} not found")

    def run(self):
        while self.running:
            queue_size = self.queue.qsize()
            tasks = []
            future_results = []
            for i in range(min(queue_size, self.num_threads)):
                tasks.append(self.queue.get())
            if tasks.__len__() > 0:
                logger.debug(f"Executing {tasks.__len__()} tasks")
                with ThreadPoolExecutor(max_workers=tasks.__len__()) as executor:
                    for task in tasks:
                        future = executor.submit(self.execute, task.id)
                        task.status = TaskStatus.RUNNING
                        future_results.append(dict(future=future, id=task.id))

                    executor.shutdown(wait=True)

                    for future in future_results:
                        self.tasks[future['id']].result = future['future'].result() if future['future'].exception() is None else future['future'].exception()
                        self.tasks[future['id']].status = TaskStatus.COMPLETED if future['future'].exception() is None else TaskStatus.FAILED

                logger.debug(f"Finished executing {tasks.__len__()} tasks")
            else:
                time.sleep(1)


RCN_EVENTS_ENABLE = "rcn.events.enable"
RCN_EVENTS_NUM_THREADS = "rcn.events.num.threads"

RCN_FLASK_APP_ENABLED = "app.flask.enabled"
RCN_FLASK_APP_NAME = "app.flask.name"
RCN_FLASK_APP_PORT = "app.flask.port"
RCN_FLASK_APP_LISTEN = "app.flask.listen"
RCN_FLASK_APP_CONTEXT_URL = "app.flask.contextUrl"
RCN_FLASK_APP_CONFIGS_LIST = "app.flask.configs"
RCN_FLASK_APP_BACKEND_SERVER = "app.flask.backend"

RCN_AUTO_CONFIGURE_SECRETS_ENABLE = "app.autoconfiguration.secrets.enable"
RCN_ENABLE_VAULT_PROVIDER = "secrets.provider.vault.enable"

RCN_ENABLE_STANDALONE_MESSAGE_BROKER = "rcn.message.brokers.standalone.enable"
RCN_ENABLE_CONTEXT_THREADS = "management.context.thread.enable"

RCN_ENABLE_TASK_EXECUTOR = "task.executor.enable"
RCN_TASK_EXECUTOR_NUM_THREADS = "task.executor.num.threads"


import logging
import os

from cndi.annotations import Component, ConditionalRendering
from cndi.autoconfiguration.configure import AutoConfigurationProviders
from cndi.consts import RCN_ENABLE_VAULT_PROVIDER
from cndi.env import getContextEnvironment, getContextEnvironments, RCN_ENVS_CONFIG, \
    reload_envs

logger = logging.getLogger(__name__)
VAULT_PROVIDER_PREFIX = "vault://"

@Component
@ConditionalRendering(callback=lambda x: getContextEnvironment(RCN_ENABLE_VAULT_PROVIDER, defaultValue=False, castFunc=bool))
class VaultSecretProvider:
    def __init__(self):
        try:
            import hvac
            vault_addr = getContextEnvironment("secrets.provider.vault.addr", "http://127.0.0.1:8200")
            vault_token = getContextEnvironment("secrets.provider.vault.token")

            client = hvac.Client(url=vault_addr, token=vault_token)
            if client.is_authenticated():
                for key, value in getContextEnvironments().items():
                    self.resolve(key, value, client)
                reload_envs()
            else:
                print("Failed to authenticate with Vault.")

            AutoConfigurationProviders._PROVIDERS[VAULT_PROVIDER_PREFIX] = self
        except ImportError as ex:
            logger.error("Could not import hvac, install hvac python package")
            raise ex

    def resolve(self, key, value, client):
        if value.startswith(VAULT_PROVIDER_PREFIX):
            secret_ref = value[len(VAULT_PROVIDER_PREFIX):]
            mount_point, path, key_ref = secret_ref.split(" ")

            read_response = client.secrets.kv.read_secret_version(mount_point=mount_point, path=path)

            if read_response and 'data' in read_response:
                value = read_response['data']['data'][key_ref]
                os.environ[RCN_ENVS_CONFIG + '.' + key] = value
            else:
                logger.warning(f"Secret not found at path: {mount_point}")


import unittest

RCN_ENVS_CONFIG = 'RCN_ENVS_CONFIG'

from cndi.env import loadEnvFromFile, getContextEnvironment, VARS


class LoadEnvTest(unittest.TestCase):
    def setUp(self) -> None:
        VARS.clear()
        VARS[f"{RCN_ENVS_CONFIG}.active.profile".lower()] = "test"


    def testloadEnvFromFile(self):
        loadEnvFromFile("tests/resources/test.yml")
        self.assertEqual("test", getContextEnvironment("rcn.profile"))

    def testloadEnvWithListDatatype(self):
        loadEnvFromFile("tests/resources/test.yml")

        self.assertEqual(getContextEnvironment('mini.listdata.#1.name'), 'thereitis')
        self.assertEqual(getContextEnvironment('mini.listdata.#0.page.#0.default'), '2')



import logging
from threading import Thread

from cndi.binders.message import Message, SubscriberChannel
from cndi.binders.message.utils import extractChannelNameFromPropertyKey, MessageChannel
from cndi.env import getContextEnvironment, getContextEnvironments

logger = logging.getLogger(__name__)

try:
    import pika
    from pika import BlockingConnection

    from pika.adapters.blocking_connection import BlockingChannel
    from pika.spec import Basic


    class RabbitMQProducerBinding(MessageChannel):
        def __init__(self, connection: BlockingConnection):
            self.channel = connection.channel()
            self.topic = None

        def setTopic(self, topic):
            MessageChannel.setTopic(self, topic)
            self.channel.queue_declare(queue=self.topic)
            self.channel.exchange_declare(exchange=self.topic)
            self.channel.queue_bind(exchange=self.topic, queue=self.topic)

        def send(self, message: Message) -> None:
            self.channel.basic_publish(
                exchange=self.topic,
                routing_key=self.topic,
                body=message.message
            )

        def close(self):
            self.channel.close()


    class RabbitMQSubscriberChannel(SubscriberChannel):
        def __init__(self, channel: BlockingChannel):
            SubscriberChannel.__init__(self)
            self.channel = channel

        def setTopic(self, topic, group="", channelName=None):
            SubscriberChannel.setTopic(self, topic)
            self.channel.queue_declare(queue=topic + group)
            self.channel.queue_bind(exchange=topic, queue=topic + group, routing_key="#")


    class RabbitMQBinder:
        @staticmethod
        def name():
            return "rabbitmq"

        def __init__(self):
            self.logger = logging.getLogger('.'.join([self.__class__.__module__, self.__class__.__name__]))
            brokerUrl = getContextEnvironment("rcn.binders.message.rabbitmq.brokerUrl", defaultValue=None)
            brokerPort = getContextEnvironment("rcn.binders.message.rabbitmq.brokerPort", defaultValue=None)

            brokerUrl = getContextEnvironment("rcn.binders.message.brokerUrl", required=True, defaultValue=brokerUrl)
            brokerPort = getContextEnvironment("rcn.binders.message.brokerPort", required=True, castFunc=int,
                                               defaultValue=brokerPort)

            user = getContextEnvironment("rcn.binders.message.rabbitmq.user")
            userSecret = getContextEnvironment("rcn.binders.message.rabbitmq.userSecret")

            contextEnvs = getContextEnvironments()
            credentials = pika.PlainCredentials(user, userSecret)
            self.connection = pika.BlockingConnection(
                pika.ConnectionParameters(host=brokerUrl, port=brokerPort, credentials=credentials, heartbeat=60))
            self.rabbotmqProducerChannelBindings = filter(
                lambda key: key.startswith('rcn.binders.message.rabbitmq.producer'),
                contextEnvs)
            self.mqttConsumerChannelBindings = filter(
                lambda key: key.startswith('rcn.binders.message.rabbitmq.consumer') and key.endswith("destination"),
                contextEnvs)
            self.CHANNELS_TO_TOPIC_MAP = dict()
            self.channel = self.connection.channel()

            self.channelThread = Thread(target=self.channel.start_consuming)
            self.producers = {}
            self.consumers = {}

        def stopConsumers(self):
            self.channel.stop_consuming()

        def info(self):
            producers = []
            consumers = []
            for channel, producerBinding in self.producers.items():
                producers.append(dict(
                    channel=channel,
                    topic=producerBinding.topic
                ))

            for channel, consumerBinding in self.consumers.items():
                consumers.append(dict(
                    channel=channel
                ))
            return dict(
                producers=producers
            )

        def health(self):
            return self.channel.is_open and self.connection.is_open

        def bindSubscribers(self, CHANNELS_TO_FUNC_MAP):
            subscriptionTopics = list()
            topicConsumers = dict()
            binders = dict()

            def consumerMessage(ch: BlockingChannel, method: Basic.Deliver, properties, body):
                topicConsumers[method.exchange](body)

            for propertyKey in self.mqttConsumerChannelBindings:
                channelName = extractChannelNameFromPropertyKey(propertyKey)
                print(channelName)
                binderPath = f"rcn.binders.message.rabbitmq.consumer.{channelName}"
                consumerGroup = getContextEnvironment(f"{binderPath}.group")

                if channelName not in CHANNELS_TO_FUNC_MAP:
                    self.logger.error(f"Channel not found: {channelName}")
                    continue
                consumerBinding = RabbitMQSubscriberChannel(self.channel)
                callbackDetails = CHANNELS_TO_FUNC_MAP[channelName]
                callback = callbackDetails['func']
                consumerBinding.setOnConsumeCallback(callback)
                topicName = getContextEnvironment(propertyKey, required=True)
                routing_key = getContextEnvironment(f"{binderPath}.routing_key", defaultValue=topicName)
                subscriptionTopics.append(topicName)

                consumerBinding.setTopic(topicName, group=consumerGroup)
                if topicName in topicConsumers:
                    raise KeyError(f"Duplicate topic found {topicName} with {topicConsumers[topicName]}")

                topicConsumers[topicName] = consumerBinding
                binders[channelName] = consumerBinding

                queue = f"{topicName}.{consumerGroup}"

                self.channel.queue_bind(queue=queue, exchange=topicName, routing_key=routing_key)
                self.channel.basic_consume(queue=queue, auto_ack=True, on_message_callback=consumerMessage)

            self.consumers = binders
            return binders

        def bindProducers(self):
            binders = dict()
            for propertyKey in self.rabbotmqProducerChannelBindings:
                channelName = extractChannelNameFromPropertyKey(propertyKey)
                producerBinding = RabbitMQProducerBinding(connection=self.connection)

                topicName = getContextEnvironment(propertyKey, required=True)
                self.logger.info("Topic Name: " + topicName)
                producerBinding.setTopic(topicName)
                self.CHANNELS_TO_TOPIC_MAP[channelName] = topicName
                binders[channelName] = producerBinding
            self.producers = binders
            return binders
except ModuleNotFoundError:
    logger.warning("RabbitMQ Binder not loaded")
    RabbitMQBinder = None